#coding=utf-8
from numpy import *
import operator
import matplotlib
import matplotlib.pyplot as plt

'''
K-近邻算法
'''

def creatDataSet():
    '''
    创建数据集函数
    :return: 
    '''
    group=array([[1.0,1.1],[1.0,1.0],[0,0],[0,0.1]])
    labels=['A','A','B','B']

    return group,labels


def classify0(inX,dataSet,labels,k):

    '''
    计算距离
    :param inX: 用于分类的输入向量
    :param dataSet: 输入的训练样本集
    :param labels: 标签向量
    :param k: 用于选择最近邻居的数目
    :return: 排序好的结果集
    '''
    '''
    shape读取矩阵的长度，比如shape[0]就是读取矩阵第一维度的长度。
    它的输入参数可以使一个整数表示维度，也可以是一个矩阵。
    '''
    dataSetSize=dataSet.shape[0]
    #输入向量inX循环与输入数据集作差
    diffMat=tile(inX,(dataSetSize,1))-dataSet
    sqDiffMat=diffMat**2
    #axis=1是将一个矩阵的每一行向量相加
    sqDistances=sqDiffMat.sum(axis=1)
    #开根号才是距离
    distances=sqDistances**0.5
    #选择距离最小的k个点
    #argsort函数返回的是数组值从小到大的索引值
    sortedDistIndicies=distances.argsort()
    classCount={}
    #range(x)函数通常结合for循环一起使用,使得循环终止于x-1的位置
    for i in range(k):
        voteIlabel=labels[sortedDistIndicies[i]]
        classCount[voteIlabel]=classCount.get(voteIlabel,0)+1
    '''
        排序
        iteritems()返回iterator 他是迭代器是访问集合元素的一种方式。
        迭代器对象从集合的第一个元素开始访问，直到所有的元素被访问完结束。
        迭代器只能往前不会后退。
        key用列表元素的某个属性或函数进行作为关键字，有默认值，迭代集合中的一项
        reverse = True 降序 或者 reverse = False 升序

    '''
    sortedClassCount=sorted(classCount.iteritems(),
                            key=operator.itemgetter(1),reverse=True)

    return sortedClassCount[0][0]

# group,labels=creatDataSet()
# print classify0([0,0],group,labels,3)

def file2matrix(filename):
    '''
    该函数负责从文本文件中读取文件并解析内容
    :param filename: 文件名
    :return: 解析后生成的数据矩阵，分类向量
    '''
    fr = open(filename)
    #获取文件行数
    numberOfLines = len(fr.readlines())
    #创建m行n列的零矩阵
    returnMat = zeros((numberOfLines,3))
    #分类标签向量的数组
    classLabelVector = []
    fr = open(filename)
    index = 0
    for line in fr.readlines():
        line = line.strip()
        #去除空格
        listFromLine = line.split('\t')
        #每一行数据作为矩阵的每一行，[0:3]表示从第0列到第3列，0也可以省略
        returnMat[index,:] = listFromLine[0:3]
        #从数据源可以看出listFromLine中最后一列为分类标签
        classLabelVector.append(int(listFromLine[-1]))
        index += 1
    return returnMat,classLabelVector

def showImage(datingDataMat,datingLabels,xlabelStr,ylabelStr,index1,index2):
    '''
    为简化图像输出而定义的专门用于输出图像的函数
    :param datingDataMat: 数据集矩阵
    :param datingLabels: 数据集标签
    :param xlabelStr: X轴描述
    :param ylabelStr: Y轴描述
    :param index1: 指定数据列1
    :param index2: 指定数据列2
    :return: 
    '''
    fig = plt.figure()
    # 111表示把绘图区域分成1行*1列共1个区域，然后在区域1上创建一个轴对象
    ax = fig.add_subplot(111)

    type1_x = []
    type1_y = []
    type2_x = []
    type2_y = []
    type3_x = []
    type3_y = []

    #按照标签分类
    for i in range(len(datingLabels)):
        if datingLabels[i] == 1:
            type1_x.append(datingDataMat[i][index1])
            type1_y.append(datingDataMat[i][index2])

        if datingLabels[i] == 2:
            type2_x.append(datingDataMat[i][index1])
            type2_y.append(datingDataMat[i][index2])

        if datingLabels[i] == 3:
            #print i, '：', datingLabels[i], ':', type(datingLabels[i])
            type3_x.append(datingDataMat[i][index1])
            type3_y.append(datingDataMat[i][index2])

    type1 = ax.scatter(type1_x, type1_y, s=20, c='red')
    type2 = ax.scatter(type2_x, type2_y, s=40, c='green')
    type3 = ax.scatter(type3_x, type3_y, s=50, c='blue')

    ax.legend((type1, type2, type3), (u'不喜欢', u'魅力一般', u'极具魅力'), loc=2)

    plt.xlabel(xlabelStr)
    plt.ylabel(ylabelStr)
    plt.show()

'''
test 1
'''
# datingDataMat,datingLabels=file2matrix('datingTestSet2.txt')
# xlabelStr=u"玩视频游戏所耗时百分百"
# ylabelStr=u"每周消耗的冰淇淋公升数"
# showImage(datingDataMat,datingLabels,xlabelStr,ylabelStr,1,2)

def autoNorm(dataSet):
    '''
    该函数用于数据归一化，用于处理不同取值范围的特征值相差较大时消除对计算结果的影响，
    将取值范围处理为0到1或-1到-1之间的数值
    :param dataSet: 
    :return: 
    '''
    #参数为0则从列中选择最小值，而不是选取当前行的最小值
    minVals=dataSet.min(0)
    maxVals=dataSet.max(0)
    ranges=maxVals-minVals
    normDataSet=zeros(shape(dataSet))
    m=dataSet.shape[0]
    #tile函数将变量内容复制成输入矩阵相同大小的矩阵
    normDataSet=dataSet-tile(minVals,(m,1))
    normDataSet=normDataSet/tile(ranges,(m,1))
    return normDataSet,ranges,minVals

# def datingClassTest():
#     '''
#     该函数用于测试算法的正确性
#     :return:
#     '''
#     #通常提供90%的数据作为样本进行训练，其余10%的数据去测试分类器，要注意10%的测试数据需要随机选择
#     hoRatio = 0.10
#     datingDataMat,datingLabels = file2matrix('datingTestSet2.txt')
#     normMat, ranges, minVals = autoNorm(datingDataMat)
#     m = normMat.shape[0]
#     numTestVecs = int(m*hoRatio)
#     errorCount = 0.0
#     for i in range(numTestVecs):
#         classifierResult = classify0(normMat[i,:],normMat[numTestVecs:m,:],datingLabels[numTestVecs:m],3)
#         print "the classifier came back with: %d, the real answer is: %d" % (classifierResult, datingLabels[i])
#         if (classifierResult != datingLabels[i]): errorCount += 1.0
#     print "the total error rate is: %f" % (errorCount/float(numTestVecs))
#     print errorCount
#     pass

def classifyPerson():
    resultList=['not at all','in small doses','in large doses']
    percentTats=float(raw_input("percentage of time spent playing video games?"))
    ffMiles=float(raw_input("frequent flier miles earned per year?"))
    iceCream=float(raw_input("liters of ice cream consumed per year?"))
    datingDataMat,datingLabels=file2matrix("datingTestSet2.txt")
    normMat,ranges,minVals=autoNorm(datingDataMat)
    inArr=array([ffMiles,percentTats,iceCream])
    classifierResult=classify0((inArr-minVals)/ranges,normMat,datingLabels,3)
    print "you will probably like this person:",resultList[classifierResult-1]
#classifyPerson()